@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 - 2016 René Rebe, ExactCODE GmbH
+ * Copyright (C) 2006 - 2017 René Rebe, ExactCODE GmbH
  *           (C) 2006, 2007 Archivista GmbH, CH-8042 Zuerich
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -425,7 +425,7 @@ bool JPEGCodec::writeImage (std::ostream* stream, Image& image, int quality,
   
   // if the instance is freestanding it can only be called by the mux
   // if the cache is valid
-  if (_image && !args.containsAndRemove("recompress")) {
+  if (!args.containsAndRemove("recompress") &&_image) {
     // if meta information was modified re-encode the stream
     if (image.isMetaModified()) {
       if (debug)
