@@ -1,6 +1,6 @@
 /*
  * C++ TGA library.
- * Copyright (C) 2008 - 2016 René Rebe, ExactCODE GmbH Germany
+ * Copyright (C) 2008 - 2017 René Rebe, ExactCODE GmbH Germany
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,10 +52,8 @@ typedef struct
   EndianessConverter<uint16_t,LittleEndianTraits> ImageHeight;
   uint8_t ImageDepth;
   uint8_t ImageDescriptor;
-  
 } __attribute__((packed)) TGAHeader;
 
-
 typedef struct
 {
   EndianessConverter<uint32_t,LittleEndianTraits> ExtensionOffset;
@@ -67,8 +65,10 @@ typedef struct
 } __attribute__((packed)) TGAFooter;
 
 
-int TGACodec::readImage(std::istream* stream, Image& image, const std::string& decompres)
+int TGACodec::readImage(std::istream* stream, Image& image, const std::string& decompress)
 {
+  Args args(decompress);
+  const bool debug = args.containsAndRemove("debug");
   bool rle = false;
 
   /*
@@ -99,6 +99,7 @@ int TGACodec::readImage(std::istream* stream, Image& image, const std::string& d
     case 32:
       break;
     default:
+      if (debug) std::cerr << "TGA: unknown depth" << std::endl;
       goto no_tga;
     };
 
@@ -121,13 +122,15 @@ int TGACodec::readImage(std::istream* stream, Image& image, const std::string& d
       break;
       
     default:
+      if (debug) std::cerr << "TGA: unknown type" << std::endl;
       goto no_tga;
     }
 
-  std::cerr << "TGA: " << (int)header.IDLength << ", " << (int)header.ImageType
-            << ", " << (int)header.ImageDepth << ", " << (int)header.ColorMapType
-	    << ", " << header.ImageWidth << ", " << header.ImageHeight
-	    << ", " << header.ImageDescriptor << std::endl;
+  if (debug)
+    std::cerr << "TGA: " << (int)header.IDLength << ", " << (int)header.ImageType
+	      << ", " << (int)header.ImageDepth << ", " << (int)header.ColorMapType
+	      << ", " << header.ImageWidth << ", " << header.ImageHeight
+	      << ", " << header.ImageDescriptor << std::endl;
   
   if (header.ImageDepth == 32)
     image.spp = 4;
@@ -137,49 +140,48 @@ int TGACodec::readImage(std::istream* stream, Image& image, const std::string& d
   
   image.resize(header.ImageWidth, header.ImageHeight);
   
-  if (header.ColorMapType == 1)
-    {
-      // seek to color map
-      stream->seekg(sizeof(header) + header.IDLength);
-    }
-  
+  if (header.ColorMapType == 1) {
+    // seek to color map
+    stream->seekg(sizeof(header) + header.IDLength);
+  }
+
   // seek to image data
   stream->seekg(sizeof(header) + header.IDLength);
+  // TODO: RGB is stored BGR
   
   if (!rle) {
-    // TODO: interpret flip/flop bits
-    stream->read((char*)image.getRawData(), image.stride() * image.h);
+    const bool topdown = (header.ImageDescriptor >> 5) & 1;
+    int beg = topdown ? 0 : image.h - 1, end = topdown ? image.h : -1,
+      inc = topdown ? 1 : -1;
+    for (int y = beg; y != end; y += inc)
+      stream->read((char*)image.getRawData() + y * image.stride(), image.stride());
   } else {
     const int bytes = header.ImageDepth / 8;
     uint8_t* data = image.getRawData();
-    uint8_t payload[4];
     
-    // TODO: I got one RLE which shows up RGB reversed?!
+    // new spec says RLE "should" not overrun scanlines, but old spec did
     for (int i = 0; i < image.stride() * image.h;)
       {
-	uint8_t t = stream->get();
+	int t = stream->get();
 	uint8_t n = (t & 0x7f) + 1;
-	if (t & 0x80) // RLE
-	  {
-	    stream->read((char*)payload, bytes);
-	    while (n-- > 0 && i < image.stride() * image.h)
-	      for (int j = 0; j < bytes; ++j)
-		data[i++] = payload[j];
-	  }
-	else // RAW
-	  {
-	    stream->read((char*)data + i, n * bytes);
-	    i += n * bytes;
-	  }
+	const bool rle = t & 0x80;
+        if (t == EOF) {
+	  std::cerr << "tga: eof" << std::endl;
+	  return true; // false;
+        }
+	if (rle) { // RLE
+	  stream->read((char*)data + i, bytes);
+	  i += bytes;
+	  for (--n; n > 0 && i < image.stride() * image.h; --n)
+	    for (int j = 0; j < bytes; ++j, ++i)
+	      data[i] = data[i - bytes];
+	} else { // RAW
+	  stream->read((char*)data + i, n * bytes);
+	  i += n * bytes;
+	}
       }
   }
   
-  {
-    unsigned ori = (header.ImageDescriptor >> 4) & 2;
-    if (ori != 2)
-      std::cerr << "unimplemented TGA orientation: " << ori << std::endl;
-  }
-  
   return true;
   
  no_tga:
@@ -190,32 +192,109 @@ int TGACodec::readImage(std::istream* stream, Image& image, const std::string& d
 bool TGACodec::writeImage(std::ostream* stream, Image& image, int quality,
 			  const std::string& compress)
 {
-  TGAHeader header;
-  header.IDLength = 0;
-  header.ColorMapType = 0;
-  header.ImageType = image.spp == 1 ? 3 /*Truecolor*/ : 2/*bw*/;
-  header.ColorMapIndex = 0;
-  header.ColorMapLength = 0;
-  header.ColorMapEntrySize = 0;
+  Args args(compress);
+  const bool rle = args.containsAndRemove("rle");
   
-  header.ImageXOrigin = 0;
-  header.ImageYOrigin = 0;
+  if (image.bps != 8) {
+    std::cerr << "TGA: unsupported bps: " << (int)image.bps << std::endl;
+    return false;
+  }
+      
+  TGAHeader header = {};
+  //header.IDLength = 0;
+  //header.ColorMapType = 0;
+  if (!rle)
+    header.ImageType = image.spp == 1 ? 3 /*Truecolor*/ : 2/*bw*/;
+  else
+    header.ImageType = image.spp == 1 ? 11 /*bw*/ : 10/*Truecolor*/;
+  //header.ColorMapIndex = 0;
+  //header.ColorMapLength = 0;
+  //header.ColorMapEntrySize = 0;
+  //header.ImageXOrigin = 0;
+  //header.ImageYOrigin = 0;
   header.ImageWidth = image.width();
   header.ImageHeight = image.height();
   header.ImageDepth = image.spp * image.bps;
   header.ImageDescriptor = 0x20; // top-left
-  
   stream->write((char*)&header, sizeof(header));
   
-  stream->write((char*)image.getRawData(), image.stride() * image.height());
-  
-  TGAFooter footer;
-  footer.ExtensionOffset = 0;
-  footer.DeveloperOffset = 0;
+  struct accessor {
+    accessor(uint8_t* ptr, const uint8_t bytes)
+      : ptr(ptr), bytes(bytes) {
+    }
+    
+    uint32_t get(unsigned int i) {
+      uint32_t v = 0;
+      uint8_t* data = ptr + i * bytes;
+      for (uint8_t j = 0; j < bytes; ++j)
+	v = v << 8 | data[j];
+      return v;
+    }
+    
+    void write(std::ostream* stream, int i, int n) {
+      for (; n > 0; --n) {
+	uint32_t v = get(i++);
+	for (int j = bytes; j > 0; --j) {
+	  *stream << (char)(v & 0xff);
+	  v >>= 8;
+	}
+      }
+    }
+    
+    uint8_t* ptr;
+    uint8_t bytes;
+  };
+
+  const unsigned stride = image.stride();
+  for (int y = 0; y < image.height(); ++y) {
+    uint8_t* data = image.getRawData() + y * stride;
+    accessor row(data, image.spp);
+    if (!rle) {
+      row.write(stream, 0, image.w);
+    } else {
+      // scan the whole stride - find repeating run
+      for (int i = 0; i < image.w;) {
+	int j;
+	for (j = i + 1; j < image.w; ++j) {
+	  if (row.get(j - 1) == row.get(j)
+	      //y&& j + 1 < image.w && row.get(j + 1) == row.get(j)
+	      ) {
+	    --j; // first repeating
+	    break;
+	  }
+	}
+	
+	// store non-repeating:
+	for (unsigned rawlen = j - i; rawlen > 0;) {
+	  unsigned len = rawlen < 0x80 ? rawlen : 0x80;
+	  *stream << (char)(len - 1); // high bit not set for uncompressed
+	  row.write(stream, i, len);
+	  i += len;
+	  rawlen -= len;
+	}
+	// find length of repeating run
+	for (i = j; j < image.w; ++j) {
+	  if (row.get(i) != row.get(j))
+	    break;
+	}
+	
+	for (int runlen = j - i; runlen > 1;) {
+	  unsigned len = runlen < 0x80 ? runlen : 0x80;
+	  *stream << (char)(0x80 | (len - 1));
+	  row.write(stream, i, 1);
+	  i += len;
+	  runlen -= len;
+	}
+      }
+    }
+  }
   
+  TGAFooter footer = {};
+  //footer.ExtensionOffset = 0;
+  //footer.DeveloperOffset = 0;
   memcpy(footer.Signature, "TRUEVISION-XFILE", sizeof(footer.Signature));
   footer.ReservedCharacter = '.';
-  footer.ZeroTerminator = 0;
+  //footer.ZeroTerminator = 0;
   stream->write((char*)&footer, sizeof(footer));
   
   return true;
