@@ -79,7 +79,7 @@ void skip_comments (std::istream* stream)
     }
 }
 
-int XPMCodec::readImage (std::istream* stream, Image& image, const std::string& decompres)
+int XPMCodec::readImage (std::istream* stream, Image& image, const std::string& decompress)
 {
   // check signature
   std::string line;
