@@ -1,6 +1,6 @@
 /*
  * The ExactImage library's displayy compatible command line frontend.
- * Copyright (C) 2006 - 2016 René Rebe, ExactCODE GmbH
+ * Copyright (C) 2006 - 2017 René Rebe, ExactCODE GmbH
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +51,8 @@ using std::cout;
 using std::cerr;
 using std::endl;
 
+static bool _root;
+
 static uint32_t evas_bgr_image_data[] = {
   0x999999, 0x666666,
   0x666666, 0x999999};
@@ -67,6 +69,30 @@ int Viewer::Window2ImageY (int y)
 
 void Viewer::Zoom (double f)
 {
+  if (_root) {
+    // TODO: add fill, center, aspect options
+    evas_image->SmoothScale (true);
+    int img_w = evas_image->Width(), img_h = evas_image->Height();
+    double sx = evas->OutputWidth(), sy = evas->OutputHeight();
+    // keep aspect
+    sx /= img_w;
+    sy /= img_h;
+    if (sx >= sy) {
+      img_w = evas->OutputWidth();
+      img_h = sx * img_h;
+    } else {
+      img_w = sy * img_w;
+      img_h = evas->OutputHeight();
+    }
+    evas_image->Resize (img_w, img_h);
+    evas_image->ImageFill(0, 0, img_w, img_h);
+    
+    // center
+    evas_image->Move((img_w - evas->OutputWidth()) / -2,
+		     (img_h - evas->OutputHeight()) / -2);
+    return;
+  }
+  
   // to keep the view centered
   int xcent = Window2ImageX (evas->OutputWidth()/2);
   int ycent = Window2ImageY (evas->OutputHeight()/2);
@@ -133,9 +159,9 @@ void Viewer::Move (int _x, int _y)
     (*it)->Move (x, y);
 }
 
-
 void Viewer::UpdateOSD (const std::string& str1, const std::string& str2)
 {
+  if (_root) return;
   evas_osd_text1->Text (str1);
   evas_osd_text2->Text (str2);
   evas_osd_rect->Resize (std::max(evas_osd_text1->Width(), evas_osd_text2->Width()) + 8*HiDPI,
@@ -170,6 +196,54 @@ void Viewer::SetOSDZoom ()
   UpdateOSD (s1.str(), s2.str());
 }
 
+#include <X11/Xatom.h>
+
+static bool setRootAtoms (Display* display, int screen, Pixmap pixmap)
+{
+  Atom atom_root, atom_eroot, type;
+  unsigned char *data_root, *data_eroot;
+  int format;
+  unsigned long length, after;
+
+  atom_root = XInternAtom (display, "_XROOTMAP_ID", True);
+  atom_eroot = XInternAtom (display, "ESETROOT_PMAP_ID", True);
+
+  // doing this to clean up after old background
+  if (atom_root != None && atom_eroot != None)
+    {
+      XGetWindowProperty (display, RootWindow (display, screen),
+                          atom_root, 0L, 1L, False, AnyPropertyType,
+                          &type, &format, &length, &after, &data_root);
+      if (type == XA_PIXMAP)
+        {
+          XGetWindowProperty (display, RootWindow (display, screen),
+                              atom_eroot, 0L, 1L, False, AnyPropertyType,
+                              &type, &format, &length, &after, &data_eroot);
+          if (data_root && data_eroot && type == XA_PIXMAP &&
+              *((Pixmap *) data_root) == *((Pixmap *) data_eroot))
+            {
+              XKillClient (display, *((Pixmap *) data_root));
+            }
+        }
+    }
+
+  atom_root = XInternAtom (display, "_XROOTPMAP_ID", False);
+  atom_eroot = XInternAtom (display, "ESETROOT_PMAP_ID", False);
+
+  if (atom_root == None || atom_eroot == None)
+    return false;
+
+  // setting new background atoms
+  XChangeProperty (display, RootWindow (display, screen),
+                   atom_root, XA_PIXMAP, 32, PropModeReplace,
+                   (unsigned char *) &pixmap, 1);
+  XChangeProperty (display, RootWindow (display, screen), atom_eroot,
+                   XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pixmap,
+                   1);
+  return true;
+}
+
+
 int Viewer::Run (bool opengl)
 {
   // TODO: move to the X11Helper ...
@@ -214,31 +288,37 @@ int Viewer::Run (bool opengl)
   int win_w = 1;
   int win_h = 1;
   
-  win = XCreateWindow (dpy, root,
-		       (X11Window::Width(dpy, 0) - win_w) / 2,
-		       (X11Window::Height(dpy, 0) - win_h) / 2,
-		       win_w, win_h,
-		       0,
-		       depth,
-		       InputOutput,
-		       visual,
-		       /*CWOverrideRedirect | */ 
-		       CWBackingStore | CWColormap |
-		       CWBackPixmap | CWBorderPixel |
-		       CWBitGravity | CWEventMask, &attr);
-
-  XStoreName (dpy, win, "edisplay");
-  chint.res_name = "edisplay";
-  chint.res_class = "Main";
-  XSetClassHint (dpy, win, &chint);
-  
+  if (_root) {
+    win_w = X11Window::Width(dpy, 0);
+    win_h = X11Window::Height(dpy, 0);
+    win = XCreatePixmap (dpy, root, win_w, win_h, depth);
+  }
+  else {
+    win = XCreateWindow (dpy, root,
+			 (X11Window::Width(dpy, 0) - win_w) / 2,
+			 (X11Window::Height(dpy, 0) - win_h) / 2,
+			 win_w, win_h,
+			 0,
+			 depth,
+			 InputOutput,
+			 visual,
+			 /*CWOverrideRedirect | */ 
+			 CWBackingStore | CWColormap |
+			 CWBackPixmap | CWBorderPixel |
+			 CWBitGravity | CWEventMask, &attr);
+    
+    XStoreName (dpy, win, "edisplay");
+    chint.res_name = "edisplay";
+    chint.res_class = "Main";
+    XSetClassHint (dpy, win, &chint);
+    
 #if 0
-  szhints.flags = PMinSize | PMaxSize | PSize | USSize;
-  szhints.min_width = szhints.max_width = win_w;
-  szhints.min_height = szhints.max_height = win_h;
-  XSetWMNormalHints(dpy, win, &szhints);
+    szhints.flags = PMinSize | PMaxSize | PSize | USSize;
+    szhints.min_width = szhints.max_width = win_w;
+    szhints.min_height = szhints.max_height = win_h;
+    XSetWMNormalHints(dpy, win, &szhints);
 #endif
-
+  }
   XSync(dpy, False);
   
   evas = new EvasCanvas ();
@@ -296,11 +376,13 @@ int Viewer::Run (bool opengl)
 #endif
 
   /* Setup */
-  evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TTF/");
-  evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TrueType/");
-  evas->FontPathPrepend ("/opt/e17/share/evas/data/");
-  evas->FontPathPrepend ("/usr/X11/share/fonts/TTF/");
- 
+  if (!_root) {
+    evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TTF/");
+    evas->FontPathPrepend ("/usr/X11/lib/X11/fonts/TrueType/");
+    evas->FontPathPrepend ("/opt/e17/share/evas/data/");
+    evas->FontPathPrepend ("/usr/X11/share/fonts/TTF/");
+  }
+  
   if (true) {
     evas->ImageCache (1024 * 1024);
     evas->FontCache (256 * 1024);
@@ -325,38 +407,55 @@ int Viewer::Run (bool opengl)
   evas_bgr_image->Show ();
   
   // OSD
-  
-  evas_osd_text1 = new EvasText (*evas);
-  evas_osd_text1->Layer (10);
-  evas_osd_text1->Move (4*HiDPI, 4*HiDPI);
-  evas_osd_text1->Font ("Vera", 10*HiDPI);
-  evas_osd_text1->Show ();
-
-  evas_osd_text2 = new EvasText (*evas);
-  evas_osd_text2->Layer (10);
-  evas_osd_text2->Move (4*HiDPI, 8*HiDPI + evas_osd_text1->Height());
-  evas_osd_text2->Font ("Vera", 10*HiDPI);
-  evas_osd_text2->Show ();
-
-  evas_osd_rect = new EvasRectangle (*evas);
-  evas_osd_rect->Layer (5);
-  evas_osd_rect->Move (2,2);
-  evas_osd_rect->Show ();
-  
-  AlphaOSD (0);
+  if (!_root) {
+    evas_osd_text1 = new EvasText (*evas);
+    evas_osd_text1->Layer (10);
+    evas_osd_text1->Move (4*HiDPI, 4*HiDPI);
+    evas_osd_text1->Font ("Vera", 10*HiDPI);
+    evas_osd_text1->Show ();
+    
+    evas_osd_text2 = new EvasText (*evas);
+    evas_osd_text2->Layer (10);
+    evas_osd_text2->Move (4*HiDPI, 8*HiDPI + evas_osd_text1->Height());
+    evas_osd_text2->Font ("Vera", 10*HiDPI);
+    evas_osd_text2->Show ();
+    
+    evas_osd_rect = new EvasRectangle (*evas);
+    evas_osd_rect->Layer (5);
+    evas_osd_rect->Move (2,2);
+    evas_osd_rect->Show ();
+    
+    AlphaOSD (0);
+  }
   
   bool image_loaded;
   image_loaded = Load ();
   if (!image_loaded)
     image_loaded = Next ();
   
-  XMapWindow (dpy, win);
-
   bool quit = false;
+  if (_root) {
+    evas->Render ();
+    setRootAtoms (dpy, scr, win);
+
+    XKillClient (dpy, AllTemporary);
+    XSetCloseDownMode (dpy, RetainTemporary);
+    
+    XSetWindowBackgroundPixmap (dpy, root, win/*pixmap*/);
+    XClearWindow(dpy, RootWindow (dpy, scr));
+    
+    XFlush (dpy);
+    XSync (dpy, False);
+    
+    quit = true;
+  } else {
+    XMapWindow (dpy, win);
+  }
+  
   Utility::Timer timer;
   int dnd_x = 0, dnd_y = 0;
   bool dnd_moved = false;
-
+  
   while (image_loaded && !quit)
     {
       // process X11 events ...
@@ -609,6 +708,8 @@ int Viewer::Run (bool opengl)
   delete evas;
   evas = 0;
   
+  XCloseDisplay(dpy);
+  
   return 0;
 }
 
@@ -686,10 +787,11 @@ bool Viewer::Load ()
   
   ImageToEvas ();
   
-  std::string title = *it;
-  title.insert (0, "edisplay: ");
-  XStoreName (dpy, win, title.c_str());
-  
+  if (!_root) {
+    std::string title = *it;
+    title.insert (0, "edisplay: ");
+    XStoreName (dpy, win, title.c_str());
+  }
   return true;
 }
 
@@ -798,6 +900,12 @@ int main (int argc, char** argv)
   Argument<bool> arg_help ("h", "help",
                            "display this help text and exit");
   arglist.Add (&arg_help);
+
+  // setup the argument list
+  Argument<bool> arg_root ("", "root",
+                           "draw image to root window pixmap and exit");
+  arglist.Add (&arg_root);
+
 #if EVASGL == 1
   Argument<bool> arg_gl ("", "gl",
 			 "Utilize OpenGL for rendering");
@@ -808,13 +916,15 @@ int main (int argc, char** argv)
   if (!arglist.Read (argc, argv) || arg_help.Get() == true || arglist.Residuals().empty())
     {
       cerr << "ExactImage viewer - edisplay, version " VERSION << std::endl
-	   << "Copyright (C) 2006 - 2016 René Rebe, ExactCODE GmbH" << std::endl
+	   << "Copyright (C) 2006 - 2017 René Rebe, ExactCODE GmbH" << std::endl
 	   << "Usage:" << endl;
       
       arglist.Usage (cerr);
       return 1;
     }
   
+  _root = arg_root.Get();
+  
   Viewer* viewer= createViewer(arglist.Residuals());
   
   int ret = viewer->Run (
